package com.ekingwin.bas.cloud.document.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;

import com.ekingwin.bas.cloud.document.config.DocumentDefineConfig;
import com.ekingwin.bas.cloud.document.dao.IDocFileDao;
import com.ekingwin.bas.cloud.document.dao.entity.DocFile;
import com.ekingwin.bas.cloud.document.dao.entity.DocumentInfo;
import com.ekingwin.bas.cloud.document.dto.UploadUserDto;
import com.ekingwin.bas.cloud.document.service.IDocFileService;
import com.ekingwin.bas.cloud.document.service.IUserService;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.DateUtil;
import com.ekingwin.bas.cloud.infra.utils.file.FileUtils;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;

@Primary
@Service
public class DocFileServiceImpl extends BaseService  implements IDocFileService{
	
	@Autowired
	private IDocFileDao docFileDao;
	@Autowired
	private DocumentDefineConfig config;
	@Autowired
	@Qualifier(value = "userServiceImpl")
	private IUserService userManager;
	
	public void save(DocFile docFile) {
		docFileDao.insertSelective(docFile);
	}

	public void delete(String id) {
		docFileDao.deleteByPrimaryKey(id);
	}

	
	public DocFile get(String id) {
		return docFileDao.selectByPrimaryKey(id);
	}

	
	private DocFile storeFile(String fileLocation) {
		File file = new File(fileLocation);
		String fileName = file.getName();
		
		String ext = "";
		int extPos = file.getName().lastIndexOf(".");
		if(extPos != -1){
			ext = fileName.substring(extPos + 1, fileName.length());
		}
		
		String fileGuid = UUID.randomUUID().toString();
		
		String baseDir = config.getDocStoreLocation();
		String newDir = FileUtils.getSafetyPath(baseDir) + DateUtil.formatDate(new Date().getTime(), "yyyyMMdd");;
		
		File dir = new File(newDir);
		if (!dir.exists()) {
			log.debug("鐩綍\"{}\"涓嶅瓨鍦紝鍒涘缓鏂扮洰褰曘�");
			dir.mkdirs();
		} else {
			log.debug("鐩綍\"{}\"宸插瓨鍦紝浣跨敤璇ョ洰褰曘�");
		}

		File storedfile = new File(FileUtils.getSafetyPath(newDir) + fileGuid);
		
		try {
			FileCopyUtils.copy(file, storedfile);
		} catch (IOException e) {
			log.error("", e);
		}
		
		DocFile df = new DocFile();
		//封装文件属性
		df.setExtension(ext);
		df.setFileSize(file.length());
		df.setNewName(fileGuid);
		df.setLocation(newDir);
		return df;
	}
	


	public DocFile storeFile(DocumentInfo doc) {
		DocFile df = this.storeFile(doc.getFileLocation());
		String ext = "";
		String fileName = doc.getFileOriginalName();
		if(StringUtils.isEmpty(fileName)){
			fileName = doc.getFileLocation();
		}
		int extPos = fileName.lastIndexOf(".");
		if(extPos != -1){
			ext = fileName.substring(extPos + 1, fileName.length());
		}
		df.setExtension(ext);
		return df;
	}

	@Override
	public InputStream getFileStream(DocFile df) throws FileNotFoundException {

		String baseDir = config.getDocStoreLocation();
		String newDir = FileUtils.getSafetyPath(baseDir) + DateUtil.formatDate(df.getCreateon().getTime(), "yyyyMMdd");
		
		String filePath = FileUtils.getSafetyPath(newDir) + df.getNewName();
		FileInputStream in = null;
		try {
			File file = new File(filePath);
			in = new FileInputStream(file);
			return in;
		} catch (FileNotFoundException e) {
			log.error("FileNotFoundException", e);
			throw new FileNotFoundException(e.getMessage());
		}
	}

	@Override
	public InputStream getFileStream(DocFile df, boolean watermark) throws FileNotFoundException {
		String baseDir = config.getDocStoreLocation();
		String newDir = FileUtils.getSafetyPath(baseDir) + DateUtil.formatDate(df.getCreateon().getTime(), "yyyyMMdd");
		
		String filePath = FileUtils.getSafetyPath(newDir) + df.getNewName();
		FileInputStream in = null;
		try {
			if(watermark)
			{
				UploadUserDto u = null;
				try {
					u = userManager.get(UserUtils.getCurrentUser());
				} catch (Exception e) {
					e.printStackTrace();
				}
				String name = "";
				String empid = "";
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
				String timeFmt = sdf.format(new Date());
				if(u != null)
				{
					name = u.getDisplayname();
					empid = u.getEmpid();
				}
				String syContext = name + " " +empid + " "+timeFmt;
				try {
					String targetPath = "";
					if(filePath.indexOf(".") != -1)
						targetPath = filePath.substring(0, filePath.lastIndexOf(".")) + "_view"+filePath.substring(filePath.lastIndexOf("."),filePath.length());
					else
						targetPath = filePath+ "_view";
				//	filePath = FileWatermark.insertWatermarkTxt(syContext, df.getExtension(), filePath, targetPath);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			File file = new File(filePath);
			in = new FileInputStream(file);
			return in;
		} catch (FileNotFoundException e) {
			log.error("FileNotFoundException", e);
			throw new FileNotFoundException(e.getMessage());
		}
	}

}
