package com.ekingwin.bas.cloud.file.service.impl;

import com.ekingwin.bas.cloud.config.FileConfig;
import com.ekingwin.bas.cloud.file.dao.IpDocumentDao;
import com.ekingwin.bas.cloud.file.dto.IpDocumentDto;
import com.ekingwin.bas.cloud.file.service.FileDocumentService;
import com.ekingwin.bas.cloud.file.vo.IpDocumentVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * 附件上传实现类
 * @author zhangjie
 * @date 2020-02-12
 */
@Primary
@Service
public class FileDocumentServiceImpl implements FileDocumentService {

    @Autowired
    private IpDocumentDao ipDocumentDao;

    @Autowired
    private FileConfig fileConfig;


    /**
     * 单个附件上传
     * @param file
     * @return
     */
    @Override
    public IpDocumentVo uploadFile(MultipartFile file) throws Exception{
        IpDocumentVo ipDocumentVo = new IpDocumentVo();
        IpDocumentDto ipDocumentDto = new IpDocumentDto();
        //获取原附件名称
        String filename=file.getOriginalFilename();
        ipDocumentDto.setOldname(filename);

        //获取文件类型
        String fileType = filename.split("\\.")[1];
        Long uuid = System.currentTimeMillis();
        //重新拼接新的文件名
        String newFileName = uuid+ "." + fileType;
        //获取文件上传的路径
        String fileDirPath=fileConfig.getCertification();
        //文件下载路径
        String fileurl = fileConfig.getFileurl();
        //判断路径是否存在
        File targetFile = new File(fileDirPath);
        if(!targetFile.exists()){
            targetFile.mkdirs();
        }
        //附件上传的全路径
        String filePath = fileDirPath+"/" + newFileName;
        //拼接附件下载路径
        String path=fileurl+"/file/file/goods/"+ newFileName;
        ipDocumentDto.setPath(path);
        ipDocumentDto.setAbsolutelyPath(filePath);
        FileOutputStream out = new FileOutputStream(filePath);
        out.write(file.getBytes());
        out.flush();
        out.close();
        //保存附件信息
        ipDocumentDao.insertDocument(ipDocumentDto);
        ipDocumentVo.setOldName(filename);
        ipDocumentVo.setId(ipDocumentDto.getId());
        ipDocumentVo.setUrl(path);
        return  ipDocumentVo;
    }

    /**
     * 多附件上传
     * @param files
     * @return
     */
    @Override
    public List<IpDocumentVo> uploadFiles(List<MultipartFile> files) throws Exception{

        List<IpDocumentVo> ipDocumentVos=new ArrayList<IpDocumentVo>();
        for (MultipartFile file : files) {
            IpDocumentVo ipDocumentVo = this.uploadFile(file);
            ipDocumentVos.add(ipDocumentVo);
        }
        return ipDocumentVos;
    }
}
