package com.ekingwin.bas.cloud.org.service.impl;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.ResultUtils;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.IOrganizationDao;
import com.ekingwin.bas.cloud.org.dao.entity.Organization;
import com.ekingwin.bas.cloud.org.dao.entity.ViewUserPost;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.dto.ViewUserPostDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.org.service.IOrganizationTenantService;
import com.ekingwin.bas.cloud.tenant.dto.TenantEmployeeDto;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;
import com.ekingwin.bas.cloud.util.StringUtils;

import tk.mybatis.mapper.entity.Example;

@Service
@Primary
public class OrganizationTenantServiceImpl extends BaseService  implements IOrganizationTenantService  {

	@Autowired
	private IOrganizationDao orgDao;
	@Autowired
	@Qualifier(value = "userMetaServiceImpl")
	private IUserMetaService userService;
	
	@Autowired
	@Qualifier(value="organizationServiceImpl")
	private IOrganizationService orgService;
	
	
	@Override
	public BaseResult<OrganizationDto> save(OrganizationDto object) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BaseResult<OrganizationDto> delete(String id) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BaseResult<PageCriteria<OrganizationDto>> query(QueryDTO query) throws OrganizationException {
		try {
			String empid = query.getAttr().get("empid");
			Example example = buildQuery(query, Organization.class);
			PageCriteria<OrganizationDto> result = query(query, () -> orgDao.getOrgByEmpid(example,empid), OrganizationDto.class);
			return ResultUtils.build(result);
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}

//		return null;
	}

	@Override
	public BaseResult<PageCriteria<UserDto>> getUserByOrg(QueryDTO query) throws BasException {
		String orgid = query.getAttr().get("orgid");
		Example example = buildQuery(query, User.class);
		PageCriteria<UserDto> result = query(query, () -> orgDao.getUserByOrg(example,orgid),
				UserDto.class);
		return ResultUtils.build(result);
	}
	
	@Override
	public BaseResult<OrganizationDto> get(String id) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean importorganization(MultipartFile file, String pid) throws BasException {
		String currentUserId = UserUtils.getCurrentUser();
		BaseResult<UserDto> BaseCurrentUserDto = userService.get(currentUserId);
		if (BaseCurrentUserDto == null || BaseCurrentUserDto.getData() == null) {
			throw new BasException("请先登陆！");
		}
		UserDto currentUserDto = BaseCurrentUserDto.getData();
		String tenantid = currentUserDto.getTenantid();
		if (tenantid == null) {
			tenantid = "0000000000";
		}
		if(StringUtils.isEmpty(pid)){
			throw new BasException("请选择父级组织！");
		}
		InputStream in = null ;
		try {
			in = file.getInputStream();
		} catch (IOException e1) {
			throw new BasException("导入发送异常，请重试!");
		}
		List<OrganizationDto> organizationDtoList = new ArrayList<OrganizationDto>();
		Workbook workBook = null;
		try {
			workBook = WorkbookFactory.create(in);
		} catch (Exception e) {
			throw new BasException("导入发送异常，请重试!");
		}
		Sheet sheet = workBook.getSheetAt(0);
		int numOfRows = sheet.getLastRowNum() + 1; // 总行数
		SimpleDateFormat sDateFormat=new SimpleDateFormat("yyyy-MM-dd");
		List<String> organizationIndexList = new ArrayList<String>();
		Map<String, OrganizationDto> organizationMap = new HashMap<String, OrganizationDto>();
		Map<String, String> organizationIndexMap = new HashMap<String, String>();
		log.info("解析excel");
		// 解析excel
		for (int i = 1; i < numOfRows; i++) {
			// i=1,表示从表格中第2行开始读取
			Row row = sheet.getRow(i);
			if (null == row) {
				continue;
			}
			OrganizationDto organizationDto = new OrganizationDto();
			Cell cell = row.getCell((short) 0); // 序号
			String index = getImportCellValue(cell);
			cell = row.getCell((short) 1); // 父级序号
			String pindex = null;
			pindex = getImportCellValue(cell);
			cell = row.getCell((short) 2);  //
			String name = getImportCellValue(cell);
			cell = row.getCell((short) 3); //
			String type = getImportCellValue(cell);
			
			if(StringUtils.isEmpty(index)){
				throw new BasException("第"+(i+1)+"行序号不能为空!");
			}else if(organizationIndexList.contains(index)){
				throw new BasException("第"+(i+1)+"行和第"+organizationIndexMap.get(index)+"行序号重复!");
			}
			if(StringUtils.isEmpty(name)){
				throw new BasException("第"+(i+1)+"行组织名称不能为空!");
			}
			if(StringUtils.isEmpty(type)){
				throw new BasException("第"+(i+1)+"行组织类型不能为空!");
			}else if("部门".equals(type)){
				type = "2";
			}else if("岗位".equals(type)){
				type = "3";
			}else if("公司".equals(type)){
				type = "1";
			}else{
				throw new BasException("第"+(i+1)+"行组织类型错误,请选择公司、部门、岗位");
			}
			organizationDto.setId((i+1)+"");
			organizationDto.setName(name);
			organizationDto.setType(type);
			organizationDto.setFullname(name);
			organizationDto.setPid(pindex);
			organizationDto.setIsdelete("N");
			organizationDtoList.add(organizationDto);
			organizationIndexMap.put(index, (i+1)+"");
			organizationIndexList.add(index);
			organizationMap.put(index, organizationDto);
			
		}
		log.info(organizationDtoList.size()+"=====================================");
		for(int i = 0 ; i < organizationDtoList.size() ; i++ ){
			OrganizationDto organizationDto = organizationDtoList.get(i);
			String pindex = organizationDto.getPid();
			if(!StringUtils.isEmpty(pindex)){
				if(!organizationIndexList.contains(pindex)){
					throw new BasException("第"+(i+2)+"行父级序号不存在!");
				}else{
					OrganizationDto porganizationDto = organizationMap.get(pindex);
					String type = porganizationDto.getType();
					if("3".equals(type)){
						String index = porganizationDto.getId();
						throw new BasException("第"+index+"行岗位下不能添加子组织!");
					}
					List<OrganizationDto> childs = porganizationDto.getChilds();
					if(childs==null){
						childs = new ArrayList<OrganizationDto>();
						porganizationDto.setChilds(childs);
					}
					childs.add(organizationDto);
				}
			}
		}
		// 保存数据
		for(int i = 0 ; i < organizationDtoList.size() ; i++ ){
			OrganizationDto organizationDto = organizationDtoList.get(i);
			String pindex = organizationDto.getPid();
			if(StringUtils.isEmpty(pindex)){
				organizationDto.setPid(pid);
				organizationDto.setId(null);
				orgService.save(organizationDto);
				saveChildsOrg(organizationDto);
				
			}
		}
		return false;
	}
	
	private String getImportCellValue(Cell cell) {
		if(cell==null){
			return "";
		}
		cell.setCellType(1);
		if(cell != null) {
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			return cell.getStringCellValue();
		}
		return null;
	}
	/**
	 * 保存子组织
	 * @param organizationDto
	 */
	private void saveChildsOrg(OrganizationDto organizationDto){
		List<OrganizationDto> childs = organizationDto.getChilds();
		String pid = organizationDto.getId();
		log.info("pid====================="+pid);
		if(childs!=null){
			for(int i = 0 ; i < childs.size() ; i++ ){
				OrganizationDto childOrganizationDto = childs.get(i);
				childOrganizationDto.setPid(pid);
				try {
					childOrganizationDto.setId(null);
					orgService.save(childOrganizationDto);
					saveChildsOrg(childOrganizationDto);
				} catch (BasException e) {
				}
			}
		}
	}
	
}
