package com.ekingwin.bas.cloud.org.service.impl;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.ResultUtils;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.IOrganizationUserDao;
import com.ekingwin.bas.cloud.org.dao.entity.OrganizationTree;
import com.ekingwin.bas.cloud.org.dao.entity.OrganizationUser;
import com.ekingwin.bas.cloud.org.dto.OrganizationUserDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationUserService;
import com.ekingwin.bas.cloud.org.vo.OrganizationCompanyVo;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;

import java.util.List;


@Service
@Primary
public class OrganizationUserServiceImpl extends BaseService implements IOrganizationUserService {

	private Logger logger = LoggerFactory.getLogger(OrganizationUserServiceImpl.class);


	@Autowired
	private IOrganizationUserDao orgUserDao;
	@Autowired
	@Qualifier(value = "userMetaServiceImpl")
	private IUserMetaService userService;
	
	@Override
	public BaseResult save(OrganizationUserDto organizationUserDto) throws BasException {
		BaseResult br=ResultUtils.build(organizationUserDto);
		try {
			OrganizationTree org=orgUserDao.queryOrg(organizationUserDto.getOrgid());
			logger.info(org.toString());
			OrganizationUser ou=orgUserDao.getOrgUser(organizationUserDto.getUserid(),organizationUserDto.getOrgid());
			//该组织下用户已经存在
			if(ou!=null){
				br.setMessage("该组织下该用户已经存在,不能重复添加!");
			}else{
				//组织只有是岗位的时候才能添加用户
				if("3".equals(org.getType())){
					save(organizationUserDto,OrganizationUser.class,orgUserDao);
				}else{
					br.setMessage("组织必须是岗位才能添加用户!");
				}
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
		return ResultUtils.build(br);
	}

	@Override
	public BaseResult<OrganizationUserDto> delete(String id) throws BasException {
		if(StringUtils.isEmpty(id)){
			throw new OrganizationException("主键ID不能为空");
		}
		orgUserDao.deleteByPrimaryKey(id);
		return ResultUtils.build();
	}

	@Override
	public BaseResult<PageCriteria<OrganizationUserDto>> query(QueryDTO query) throws BasException {
		// TODO Auto-generated method stub
		try {
			Example example = buildQuery(query, OrganizationUser.class);
			PageCriteria<OrganizationUserDto> result = query(query, 
						() -> orgUserDao.selectByExample(example), 
						OrganizationUserDto.class);
			return ResultUtils.build(result);			
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}
	@Override
	public BaseResult<OrganizationUserDto> get(String id) throws BasException {
		try {
			OrganizationUserDto result = get(OrganizationUserDto.class, id, orgUserDao);
			return ResultUtils.build(result);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e.getMessage(),e);
		}
	}
	@Override
	public BaseResult deleteAll() throws OrganizationException{
		
		return ResultUtils.build();
	}

	@Override
	public OrganizationUserDto getOrgUser(String userid, String orgid) throws OrganizationException {
		// TODO Auto-generated method stub
		OrganizationUser orguser = orgUserDao.getOrgUser(userid, orgid);
		OrganizationUserDto orguserDto =new OrganizationUserDto();
		try {
			commonBuild(orguserDto, orguser);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new OrganizationException(e.getMessage(),e);
		}
		return orguserDto;
	}

	@Override
	public OrganizationUserDto insertOrupdate(String empid, String orgid,String ismainpost) throws OrganizationException {

		User user=userService.getUserByEmpid(empid);
		String userid=user.getEmpuid();
		OrganizationUser organizationUser= new OrganizationUser();
		
		if(StringUtils.isNotEmpty(userid)) {
			organizationUser.setUserid(userid);
			organizationUser.setOrgid(orgid);
			organizationUser.setIsmainpost(ismainpost);
			OrganizationUser orguser = orgUserDao.getOrgUser(userid, orgid);
			if(null !=orguser) {
				organizationUser.setId(orguser.getId());
				orgUserDao.updateByPrimaryKeySelective(organizationUser);
			}else {
				orgUserDao.insertSelective(organizationUser);
			}
		}
		
		return null;
	}

	@Override
	public OrganizationUser getOrganizationUser(String empid, String orgid, String ismainpost)
			throws OrganizationException {
		/*User user=userService.getUserByEmpid(empid);
		String userid=user.getEmpuid();*/
		String userid=userService.getEmpuidByEmpid(empid);
		OrganizationUser organizationUser= new OrganizationUser();
		
		if(StringUtils.isNotEmpty(userid)) {
			organizationUser.setUserid(userid);
			organizationUser.setOrgid(orgid);
			organizationUser.setIsmainpost(ismainpost);
			String id=orgUserDao.getOrgUserId(userid, orgid);
			//OrganizationUser orguser = orgUserDao.getOrgUser(userid, orgid);
			if(null !=id) {
				organizationUser.setId(id);	
			}
		}
		return organizationUser;
	}

	@Override
	public void batchUpdate(List<OrganizationUser> updateList) throws OrganizationException {
		// TODO Auto-generated method stub
		orgUserDao.batchUpdate(updateList);
	}

	@Override
	public void batchInsert(List<OrganizationUser> insertList) throws OrganizationException {
		// TODO Auto-generated method stub
		orgUserDao.batchInsert(insertList);
	}

	@Override
	public boolean deleteUser(String userid, String orgid) throws OrganizationException {
		orgUserDao.deleteUser(userid,orgid);
		return true;
	}

	/**
	 * 根据工号获取当前
	 * @param userid
	 * @return
	 */
	@Override
	public  List<OrganizationCompanyVo> getCompanyByUserId(String userid){

		return  orgUserDao.getCompanyByUserId(userid);
	}
}
