package com.ekingwin.bas.cloud.org.web;


import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.dto.SaveResultDTO;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.infra.web.BaseController;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.dto.OrganizationTreeDto;
import com.ekingwin.bas.cloud.org.dto.OrganizationTreePostDto;
import com.ekingwin.bas.cloud.org.dto.ViewUserPostDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.org.service.IViewUserPostService;
import com.ekingwin.bas.cloud.user.UserException;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.UUID;


@RestController
@RequestMapping("/org/organization")
@Api("用户OrganizationController相关api")
public class OrganizationController extends BaseController{
	@Autowired
	@Qualifier(value="organizationServiceImpl")
	private IOrganizationService orgService;
	
	@Autowired
	@Qualifier(value="viewUserServiceImpl")
	private IViewUserPostService userPostService;
	
	/**
	 * 查询所有组织 
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/query", method = RequestMethod.POST)
	public BaseResult<PageCriteria<OrganizationDto>> query(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgService.query(query);
	}
	
	/**
	 * 保存组织
	 * @param orgDto
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	public SaveResultDTO save(@RequestBody OrganizationDto orgDto) throws Exception {
		SaveResultDTO result = new SaveResultDTO();
		try {
			orgDto.setIsdelete("N");
			if(StringUtils.isEmpty(orgDto.getId())){
				orgDto.setId(UUID.randomUUID().toString());
			}
			orgService.save(orgDto);
			result.setResult("1");
			result.setMessage(orgDto.getId());
		} catch (OrganizationException e) {
			result.setResult("500");
			result.setMessage(e.getMessage());
		}
		return result;
	}
	
	/**
	 * 根据id删除组织
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/delete/{Id}", method = RequestMethod.POST)
	public SaveResultDTO delete(@PathVariable(value = "Id")String Id) throws Exception {
		
		SaveResultDTO result = new SaveResultDTO();
		try {
		    orgService.deleteById(Id);
			result.setResult("1");
			result.setMessage(Id);
		} catch (OrganizationException e) {
			result.setResult("0");
			result.setMessage(e.getMessage());
		}
		return result;
	}
	
	/**
	 * 根据id查询组织
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/get/{id}", method = RequestMethod.GET)
	public BaseResult<OrganizationDto> get(@PathVariable(value = "id")String Id) throws Exception {
		return orgService.get(Id);
	}
	
	/**
	 * 获取指定用户的部门信息
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/dept/{id}", method = RequestMethod.GET)
	public OrganizationDto getDept(@PathVariable(value = "id")String userId) throws Exception {
		return orgService.getUserDept(userId);
	}
	
	/**
	 * 获取指定用户的岗位信息
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/post/{id}", method = RequestMethod.GET)
	public List<OrganizationDto> getPost(@PathVariable(value = "id")String userId) throws Exception {
		return orgService.getUserPost(userId);
	}
	
	/**
	 * 获取指定用户的岗位信息（用这个）
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/queryPost", method = RequestMethod.POST)
	public PageCriteria<ViewUserPostDto> queryPost(@RequestBody(required = false) QueryDTO query) throws Exception {
		return userPostService.query(query);
	}
	
	/**
	 * 根据id返回权限操作标志
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getString/{id}", method = RequestMethod.GET)
	public String getStringById(@PathVariable(value = "id")String id) throws Exception {
		return orgService.getStringById(id);
	}
	
	/**
	 * 获取当前用户的顶级组织
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getLast", method = RequestMethod.GET)
	public List<OrganizationTreeDto> getLastTree() throws Exception {
		return orgService.getLastParentTree();
	}
	
	/**
	 * 获取第一级组织
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getFirstTree", method = RequestMethod.GET)
	public List<OrganizationTreeDto> getFirstTree() throws Exception {
		return orgService.selectFirstTree();
	}
	
	/**
	 * 根据id获取指定的子组织
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getChild/{id}", method = RequestMethod.GET)
	public List<OrganizationTreeDto> getChildById(@PathVariable(value = "id")String userId) throws Exception {
		return orgService.getChildTreeById(userId);
	
	}
	
	/**
	 * 根据id获取所有下一级组织
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getAllChild/{id}", method = RequestMethod.GET)
	public List<OrganizationTreeDto> getChildTreeById(@PathVariable(value = "id")String id) throws Exception {
		return orgService.getAllChildTree(id);
	
	}
	
	/**
	 * 获取下一级岗位
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getChildPost", method = RequestMethod.PUT)
	public List<OrganizationTreeDto> getChildPost(@RequestBody Map<String, String> map) throws Exception {
		return orgService.getChildPost(map);
	
	}
	
	@RequestMapping(value = "/getPostByOrgid", method = RequestMethod.POST)
	@ApiOperation("查询组织维度下面的岗位组织")
	public List<OrganizationTreePostDto> getPostByOrgid(@RequestBody Map<String, String> map) throws OrganizationException{
		String logid = map.get("logid");
		String orgid = map.get("orgid");
		String flag = map.get("flag");
		int num = 0;
		if (!StringUtils.isEmpty(flag)) {
			num = Integer.parseInt(flag);
		}
		if (StringUtils.isEmpty(logid)) {
			logid = UserUtils.getCurrentUser();
		}
		return orgService.getPostByOrgid(logid,orgid,num);
	}
	
	@RequestMapping(value = "/getUserByLogIdAndOrgid", method = RequestMethod.POST)
	@ApiOperation("通过组织ID和登录用户ID,获取该组织下的所有员工")
	public PageCriteria<UserDto> getUserByLogIdAndOrgid(@RequestBody(required = false) QueryDTO query) throws UserException{
		String logid = query.getAttr().get("logid");
		String orgid = query.getAttr().get("orgid");
		if (StringUtils.isEmpty(logid)) {
			logid = UserUtils.getCurrentUser();
		}
		return orgService.getUserByLogIdAndOrgid(query,logid,orgid);
	}
	
	
	@RequestMapping(value = "/getPostRouteByPostid/{postid}", method = RequestMethod.GET)
	@ApiOperation("根据岗位id获取岗位全路径")
	public String getPostRouteByPostid(@PathVariable(value = "postid")String postid){
		try {
			if (!StringUtils.isEmpty(postid)) {
				return orgService.getPostRouteByPostid(postid);
			}
		} catch (Exception e) {
			e.getMessage();
		}
		return null;
	}
	
	/**
	 * 同步用户列表
	 * @param postDto
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/syncUserList", method = RequestMethod.POST)
	public SaveResultDTO syncOrg(@RequestBody List<OrganizationDto> listOrgDto) throws Exception {
		SaveResultDTO result = new SaveResultDTO();
		 for(OrganizationDto orgDto:listOrgDto){
			 try {
					orgDto.setIsdelete("N");
				    orgService.save(orgDto);
					result.setResult("1");
					result.setMessage(orgDto.getId());
				} catch (OrganizationException e) {
					result.setResult("500");
					result.setMessage(e.getMessage());
				} 
		 }
		
		return result;
	}
	
	/**
	 * 查询组织下的用户
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getUsersByOrg", method = RequestMethod.POST)
	@ApiOperation("查询组织下的用户")
	public BaseResult<PageCriteria<UserDto>> getUsersByOrg(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgService.getUsersByOrg(query);
	}
	
}
