package com.ekingwin.bas.cloud.org.web;

import java.util.List;
import java.util.Map;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.web.BaseController;
import com.ekingwin.bas.cloud.org.dto.PostDto;
import com.ekingwin.bas.cloud.org.service.IPostService;

@RestController
@RequestMapping("/org/post")
@Api("用户postController相关api")
public class PostController extends BaseController{
	@Autowired
	@Qualifier(value="postServiceImpl")
	private IPostService postService;
	
	/**
	 * 查询所有岗位
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/query", method = RequestMethod.POST)
	public BaseResult<PageCriteria<PostDto>> query(@RequestBody(required = false) QueryDTO query) throws Exception {
		return postService.query(query);
	}
	
	/**
	 * 保存岗位
	 * @param postDto
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	public BaseResult<PostDto> save(@RequestBody PostDto postDto) throws Exception {
		return postService.save(postDto);
	}
	
	/**
	 * 根据id删除岗位
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/delete/{Id}", method = RequestMethod.DELETE)
	public BaseResult<PostDto> delete(@PathVariable(value = "Id")String Id) throws Exception {
		return postService.delete(Id);
	}
	
	/**
	 * 根据id查询岗位
	 * @param Id
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/get/{id}", method = RequestMethod.GET)
	public BaseResult<PostDto> get(@PathVariable(value = "id")String Id) throws Exception {
		return postService.get(Id);
	}
	
	@RequestMapping(value = "/getUserPost/{userId}", method = RequestMethod.GET)
	@ApiOperation("根据用户ID查询该用户属于哪些岗位的ID")
	public List<PostDto> getUserPost(@PathVariable(value = "userId")String userId) throws Exception {
		return postService.getUserPost(userId);
	}

}
