package com.ekingwin.bas.cloud.supplier.service.impl;

import com.ekingwin.bas.cloud.dto.BasePageDto;
import com.ekingwin.bas.cloud.handler.BusinessException;
import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.mail.dto.MailInfoDto;
import com.ekingwin.bas.cloud.mail.service.IMailService;
import com.ekingwin.bas.cloud.org.dao.IpSmsRecordMapper;
import com.ekingwin.bas.cloud.org.dao.entity.IpSmsRecord;
import com.ekingwin.bas.cloud.supplier.dao.IpSapSupplierDtoMapper;
import com.ekingwin.bas.cloud.supplier.dao.SupplierDao;
import com.ekingwin.bas.cloud.supplier.dto.*;
import com.ekingwin.bas.cloud.supplier.service.ISupplierService;
import com.ekingwin.bas.cloud.supplier.vo.AccountCenterInfo;
import com.ekingwin.bas.cloud.supplier.vo.SupplierBankVo;
import com.ekingwin.bas.cloud.supplier.vo.SupplierPageInfoVo;
import com.ekingwin.bas.cloud.tenant.dto.TenantEmployeeDto;
import com.ekingwin.bas.cloud.tenant.service.ITenantEmployeeService;
import com.ekingwin.bas.cloud.user.dao.UserDao;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.dto.IpcOutUserDto;
import com.ekingwin.bas.cloud.user.dto.PasswordInfoDto;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.ekingwin.bas.cloud.user.dto.UserToBpmDto;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;
import com.ekingwin.bas.cloud.user.service.IpcOutUserService;
import com.ekingwin.bas.cloud.util.*;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.util.*;

@Primary
@Service
public class SupplierServiceImpl extends BaseService implements ISupplierService {
    @Autowired
    private SupplierDao dao;

    private Logger logger = LoggerFactory.getLogger(SupplierServiceImpl.class);
    @Autowired
    private UserDao userDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private IpSmsRecordMapper ipSmsRecordMapper;

    @Autowired
    private IpcOutUserService ipcOutUserService;

    @Autowired
    @Qualifier(value = "userMetaServiceImpl")
    private IUserMetaService userService;

    @Autowired
    @Qualifier(value = "tenantEmployeeServiceImpl")
    private ITenantEmployeeService tenantEmployeeService;

    @Autowired
    @Qualifier(value = "mailServiceImpl")
    private IMailService mailService;

    @Value("${accessKeyID}")
    private String accessKeyId;
    @Value("${accessKeySecret}")
    private String secret;
    @Value("${domainUrl}")
    private String domainUrl;
    @Value("${dataSourceForUpPwd}")
    private String dataSource;
    @Value("${springMailFrom}")
    private String springMailFrom;
    @Value("${springMailFromPass}")
    private String springMailFromPass;


    @Value("${defaultpos}")
    private String defaultpos;
    @Value("${defaultposbas}")
    private String defaultposbas;
    @Value("${deptid}")
    private String deptid;
    @Value("${defaultRole}")
    private String defaultRole;
    @Value("${internalRole}")
    private String internalRole;

    @Override
    public void createUser(SupplierDto supplierDto) throws BasException {

        String unitecode = supplierDto.getUnitecode();
        List<User> userList = userDao.getEmployeeByMobilephoneEmpuidEmail(unitecode);
        if (userList.size() > 0) {
            throw new BasException("账号已存在");
        }

        try {

            TenantEmployeeDto tenantEmployeeDto = new TenantEmployeeDto();
            tenantEmployeeDto.setTenantid("000000000");
            tenantEmployeeDto.setMobilephone(supplierDto.getUnitecode());
            tenantEmployeeService.save(tenantEmployeeDto);
            tenantEmployeeDto = tenantEmployeeService.getByMobilephone(supplierDto.getUnitecode());
            String empuid = tenantEmployeeDto.getId();


            UserDto userDto = new UserDto();
            userDto.setCnname(supplierDto.getAbbreviation());
            userDto.setEmpuid(empuid);
            userDto.setMobilephone(supplierDto.getUnitecode());
            userDto.setEmpid(supplierDto.getUnitecode());
            userDto.setEmail("");
            userDto.setDisplayname(supplierDto.getFullname());
            userDto.setBusinesstype("02");
            //人员数据同步到IDAP
            Map<String, String> map = new HashMap<String, String>();
            map.put("gidNumber", "100");
            map.put("homeDirectory", "/home/admin");
            map.put("sn", userDto.getEmpuid());
            map.put("uid", userDto.getEmpuid());
            map.put("uidNumber", "200");
            map.put("mail", "");
            map.put("mobile", unitecode);
            map.put("givenName", userDto.getCnname());
            map.put("displayName", userDto.getDisplayname());
            userService.saveToLdap(userDto, map, unitecode);

            userService.save(userDto);
        } catch (Exception e) {
            throw new BasException("创建用户信息失败！");
        }

    }


    @Override
    public void forgetUserPwd(PasswordInfoDto passwordInfoDto) {
        String verificationcode = passwordInfoDto.getVerificationcode();
        String way = passwordInfoDto.getWay();
        String account = passwordInfoDto.getEmpuid();
        int platform = passwordInfoDto.getPlatForm();
        IpSmsRecord ipSmsRecord = ipSmsRecordMapper.selectCodeByPhone(way, platform);

        if (null != ipSmsRecord) {
            String code = ipSmsRecord.getCode();
            if (StringUtils.isEmpty(code)) {
                throw new BusinessException(500, "验证码不能为空");
            } else if (!code.equals(verificationcode)) {
                throw new BusinessException(500, "验证码错误");
            }
            String newPassword = passwordInfoDto.getNewpassword();
            newPassword = Md5Util.getMd5Password(newPassword);
            int result = 0;
            if (platform == 1) {
                result = userDao.updatePwdByPhoneForSup(dataSource, way, newPassword, account);
            } else if (platform == 2) {
                result = userDao.updatePwdByPhoneForPur(way, newPassword, account);
            }
        } else {
            throw new BusinessException(500, "验证码过期，请重新获取！");
        }
    }


    @Override
    public Result sendSmsCode(PasswordInfoDto passwordInfoDto) throws Exception {
        if (StringUtils.isEmpty(passwordInfoDto.getWay()) || StringUtils.isEmpty(passwordInfoDto.getWayType())) {
            return Result.error(500, "您需要填写邮箱或者手机信息！");
        }
        //短信验证
        if (passwordInfoDto.getWayType().equals("1")) {
            //密码重置模板code
            String templateCode = SmsTemplate.RESPONSE_CODE.get("PASSWORDRESETCODE");
            String signName = SmsTemplate.EGOSIGN;
            Result result = SmsUtils.SendSms(passwordInfoDto.getWay(), templateCode, signName, accessKeyId, secret, domainUrl);
            if (result.get("code").toString().equals("200")) {
                IpSmsRecord ipSmsRecord = new IpSmsRecord();
                ipSmsRecord.setUserid(passwordInfoDto.getEmpuid());
                ipSmsRecord.setPhone(result.get("phoneNumber").toString());
                ipSmsRecord.setCode(result.get("verificationCode").toString());
                ipSmsRecord.setContent(result.get("content").toString());
                ipSmsRecord.setStatus(1);
                ipSmsRecord.setPlatform(passwordInfoDto.getPlatForm());
                ipSmsRecord.setCreateDate(new Date());
                ipSmsRecordMapper.insertSelective(ipSmsRecord);
                return Result.ok();
            } else {
                return result;
            }
        } else if (passwordInfoDto.getWayType().equals("2")) {//邮箱验证
            HashMap codeParam = new HashMap();
            List<String> receivers = new ArrayList<String>();
            receivers.add(passwordInfoDto.getWay());
            List<String> list = new ArrayList<String>();
            String code = SmsUtils.getSixCode();
            codeParam.put("code", code);
            MailInfoDto mail = new MailInfoDto();
            mail.setParams(codeParam);
            mail.setReceivers(receivers);
            mail.setSender(springMailFrom);
            mail.setSenderPass(springMailFromPass);
            //mail.setContent(SmsTemplate.EGOMAILTEMPLATE.replace("${code}",code));
            mail.setSubject("融果果重置密码验证码");
            mail.setTemplate("resetPasswordTemplate.ftl");
            mail.setBcc(list);
            mail.setCc(list);
            //BaseResult baseResult = mailService.sendMail(mail);
            BaseResult baseResult = mailService.sendMailWithTemplate(mail);
            if (baseResult.getStatus().equals("200")) {
                IpSmsRecord ipSmsRecord = new IpSmsRecord();
                ipSmsRecord.setUserid(passwordInfoDto.getEmpuid());
                ipSmsRecord.setCode(code);
                ipSmsRecord.setStatus(1);
                ipSmsRecord.setPlatform(passwordInfoDto.getPlatForm());
                ipSmsRecord.setPhone(passwordInfoDto.getWay());
                ipSmsRecord.setContent(SmsTemplate.EGOMAILTEMPLATE.replace("${code}", code));
                ipSmsRecord.setCreateDate(new Date());
                ipSmsRecordMapper.insertSelective(ipSmsRecord);
                return Result.ok();
            } else {
                return Result.error(500, "邮箱发送失败，请通过其它方式验证。");
            }
        }
        return Result.error(500, "请填写验证方式");
    }


    @Override
    public Result getPhoneAndMail(PasswordInfoDto passwordInfoDto) throws Exception {
        Map<String, String> map = new HashMap<>();
        if (passwordInfoDto.getPlatForm() == 1) {//获取供应商表的手机号和邮箱
            map = userDao.getPhoneAndMailForSup(dataSource, passwordInfoDto.getEmpuid());
        } else if (passwordInfoDto.getPlatForm() == 2) {//获取采购平台用户表的手机号和邮箱
            map = userDao.getPhoneAndMailForPur(passwordInfoDto.getEmpuid());
        }
        if (null == map) {
            return Result.error(500, "您输入的账户不存在，请重新输入。");
        }
        return Result.ok(map);
    }

    @Override
    public Result updateUserPwd(PasswordInfoDto passwordInfoDto) throws Exception {
        if (StringUtils.isEmpty(passwordInfoDto.getNewpassword()) || StringUtils.isEmpty(passwordInfoDto.getOldpassword())) {
            throw new BasException("请输入新密码和旧密码。");
        }
        Map<String, Object> passMap = new HashMap<>();
        if (passwordInfoDto.getPlatForm() == 1) {
            passMap = userDao.getPasswordBySup(dataSource, passwordInfoDto.getEmpuid());
            if (null != passMap) {
                String oldpassword = passwordInfoDto.getOldpassword();
                oldpassword = Md5Util.getMd5Password(oldpassword);
                if (!oldpassword.equals(passMap.get("password"))) {
                    return Result.error(500, "原密码不对,请重新输入");
                }
                String newpassWord = passwordInfoDto.getNewpassword();
                newpassWord = Md5Util.getMd5Password(newpassWord);
                int result = userDao.updatePwdByAccountForSup(dataSource, newpassWord, passwordInfoDto.getEmpuid());
                if (result > 0) {
                    return Result.ok();
                }
            } else {
                return Result.error(500, "用户不存在");
            }
        } else if (passwordInfoDto.getPlatForm() == 2) {
            passMap = userDao.getPasswordByPur(passwordInfoDto.getEmpuid());
            if (null != passMap) {
                String oldpassword = passwordInfoDto.getOldpassword();
                oldpassword = Md5Util.getMd5Password(oldpassword);
                if (!oldpassword.equals(passMap.get("password"))) {
                    return Result.error(500, "原密码不对,请重新输入");
                }
                String newpassWord = passwordInfoDto.getNewpassword();
                newpassWord = Md5Util.getMd5Password(newpassWord);
                int result = userDao.updatePwdByAccountForPur(newpassWord, passwordInfoDto.getEmpuid());
                if (result > 0) {
                    return Result.ok();
                }
            } else {
                return Result.error(500, "用户不存在");
            }
        }
        return Result.error(500, "系统错误,请联系管理员");
    }


    @Override
    public PageInfo<SupplierPageInfoVo> selectPage(BasePageDto basePageDto) throws Exception {
        PageHelper.startPage(basePageDto.getPagenum(), basePageDto.getPagesize());
        List<SupplierPageInfoVo> supplierPageInfoVoList = supplierDao.selectPage(basePageDto);
        PageInfo<SupplierPageInfoVo> pageInfo = new PageInfo<>(supplierPageInfoVoList);
        return pageInfo;
    }

    @Override
    public AccountCenterInfo getAccountCenterInfo(String supplierId) throws Exception {
        return supplierDao.getAccountCenterInfo(dataSource, supplierId);
    }

    @Override
    public void updateAccountCenterInfo(AccountCenterInfo accountCenterInfo) throws Exception {
        if (!StringUtils.isEmpty(accountCenterInfo.getSupplierId())) {
            accountCenterInfo.setDataSource(dataSource);
            supplierDao.updateAccountCenterInfo(accountCenterInfo);
        } else {
            throw new BasException(500, "系统错误，请联系管理员");
        }
    }

    /**
     * 同步供应商
     *
     * @param suppliersDto
     */
    @Override
    public void saveOrUpdateSuppliers(SuppliersDto suppliersDto) throws Exception {
        //同步供应商
        SuppliersSaveDto suppliersSaveDto = new SuppliersSaveDto();
        suppliersSaveDto.setSupplierId(suppliersDto.getLifnr());
        suppliersSaveDto.setAccount(suppliersDto.getLifnr());
        suppliersSaveDto.setName(suppliersDto.getLifnrName());
        suppliersSaveDto.setFullname(suppliersDto.getLifnrName());
        suppliersSaveDto.setNum(suppliersDto.getLifnr());
        List<SupplierPersionDto> persions = suppliersDto.getPersions();
        if (persions != null && persions.size() > 0) {
            SupplierPersionDto supplierPersionDto = persions.get(0);
            if (supplierPersionDto !=null){
                suppliersSaveDto.setContact(supplierPersionDto.getContactName());
                suppliersSaveDto.setPhone(supplierPersionDto.getMobilePhone());
                suppliersSaveDto.setEmail(supplierPersionDto.getEmail());
            }
        }
        suppliersSaveDto.setTel(suppliersDto.getComPhone());
        suppliersSaveDto.setTaxnum(suppliersDto.getAxRegCert());
        List<SupplierBankDto> banks = suppliersDto.getBanks();
        if (banks != null && banks.size() > 0) {
            SupplierBankDto supplierBankDto = banks.get(0);
            if (supplierBankDto !=null){
                suppliersSaveDto.setSubank(supplierBankDto.getBankName());
                suppliersSaveDto.setSubanknum(supplierBankDto.getTaxRegNo());
            }

        }
        supplierDao.insertSuppliers(suppliersSaveDto);

        //插入外部用户和对供应商授权普通用户
        this.saveOutUser(suppliersSaveDto);

        //插入bpm
        this.saveBpmUser(suppliersSaveDto);
    }

    private void saveOutUser(SuppliersSaveDto suppliersSaveDto) throws Exception{
        IpcOutUserDto ipcOutUserDto=new IpcOutUserDto();
        ipcOutUserDto.setUsermsg(suppliersSaveDto.getAccount());
        ipcOutUserDto.setContacts(suppliersSaveDto.getContact());
        ipcOutUserDto.setCallPhone(suppliersSaveDto.getPhone());
        ipcOutUserDto.setEmail(suppliersSaveDto.getEmail());
        ipcOutUserDto.setCompanyTypeId("5");
        ipcOutUserDto.setCompanyType("签约供应商");
        ipcOutUserDto.setJobTitle("签约供应商");
        ipcOutUserDto.setCompany(suppliersSaveDto.getFullname());
        ipcOutUserDto.setState(1);
        ipcOutUserService.initIpCoutUser(ipcOutUserDto);
    }



    private void saveBpmUser(SuppliersSaveDto suppliersSaveDto) throws Exception {
        UserToBpmDto ubd = new UserToBpmDto();

        ubd.setEmpid(suppliersSaveDto.getAccount());
        ubd.setEmpuid(suppliersSaveDto.getAccount());
        ubd.setCnname(suppliersSaveDto.getFullname());
        ubd.setEmail(suppliersSaveDto.getEmail());
        ubd.setPhone(suppliersSaveDto.getPhone());
        ubd.setIsDeleted("N");
        ubd.setDisplayname(suppliersSaveDto.getFullname());
        ubd.setDeptid(deptid);
        ubd.setDefaultpos(Integer.valueOf(defaultposbas));
        ubd.setMobilephone(suppliersSaveDto.getPhone());
        ubd.setRemark("1");
        User user = new User();
        BeanUtils.copyProperties(ubd, user);
        userDao.saveUserToEmployee(user);
        Boolean flag = userService.saveUserToBpm(ubd);
    }

    /**
     * 获取sap银行信息
     * @param supplierId
     * @return
     */
    @Override
    public List<SupplierBankVo> getSuppliersBank(String supplierId){

        return  supplierDao.getSuppliersBank(supplierId);
    }


    @Autowired
    private IpSapSupplierDtoMapper ipSapSupplierDtoMapper;






    @Override
    public void initSapSuppliers(int pageSize,int pageNum) throws Exception{
        PageHelper.startPage(pageNum,pageSize);
        List<SuppliersDto> supplierList = ipSapSupplierDtoMapper.getSupplierList();
        PageInfo<SuppliersDto> pageInfo=new PageInfo<>(supplierList);
        List<SuppliersDto> list = pageInfo.getList();
        if (list !=null && list.size()>0){
            for (SuppliersDto suppliersDto : list) {
                String lifnr = suppliersDto.getLifnr();
                suppliersDto.setBanks(ipSapSupplierDtoMapper.getBankList(lifnr));
                suppliersDto.setPersions(ipSapSupplierDtoMapper.getContractList(lifnr));
                suppliersDto.setScopes(ipSapSupplierDtoMapper.getScopeList(lifnr));
               this.saveOrUpdateSuppliers(suppliersDto);
            }
            //获取最后一页
            int navigateLastPage = pageInfo.getNavigateLastPage();
/*            if (navigateLastPage>pageNum){
                this.initSapSuppliers(10,pageNum+1);
            }*/
        }


    }
}
