package com.ekingwin.bas.cloud.user.service.impl;

import com.ekingwin.bas.cloud.attachment.dto.FileObject;
import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.FilterDTO;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.dto.SaveResultDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.ResultUtils;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.ldap.LdapException;
import com.ekingwin.bas.cloud.ldap.dto.LdapPersonDto;
import com.ekingwin.bas.cloud.ldap.service.ILdapPersonService;
import com.ekingwin.bas.cloud.org.dao.IOrganizationManagerDao;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.rolejcy.service.IRoleegoService;
import com.ekingwin.bas.cloud.user.UserException;
import com.ekingwin.bas.cloud.user.dao.ISystemUserEsbDao;
import com.ekingwin.bas.cloud.user.dao.UserDao;
import com.ekingwin.bas.cloud.user.dao.UsersDao;
import com.ekingwin.bas.cloud.user.dao.entity.SystemUserEsb;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.dto.*;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;
import com.ekingwin.bas.cloud.user.service.IpcOutUserService;
import com.ekingwin.bas.cloud.verificationCode.dto.VerificationCodeDto;
import com.ekingwin.bas.cloud.verificationCode.service.IVerificationCodeService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import tk.mybatis.mapper.entity.Example;

import java.awt.*;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.List;

@Primary
@Service
public class UserMetaServiceImpl extends BaseService implements IUserMetaService {

	private Logger logger = LoggerFactory.getLogger(UserMetaServiceImpl.class);


	@Autowired
	private UserDao userDao;
	@Autowired
	private UsersDao usersDao;

	@Autowired
	private IOrganizationManagerDao iOrganizationManagerDao;

	@Autowired
	private IOrganizationService orgService;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private ISystemUserEsbDao systemUserEsbDao;

	@Autowired
	private IpcOutUserService ipcOutUserService;

	@Autowired
	private IRoleegoService roleegoService;

	@Autowired
	@Qualifier(value = "ldapPersonServiceImpl")
	private ILdapPersonService ldapPersonService;

	@Autowired
	@Qualifier(value = "verificationCodeServiceImpl")
	private IVerificationCodeService verificationCodeService;
	@Value("${dataSourceForSaveUser}")
	private String dataSourceForSaveUser;
	@Value("${defaultpos}")
    private String defaultpos;
	@Value("${defaultposbas}")
	private String defaultposbas;
	@Value("${deptid}")
    private String deptid;
	@Value("${defaultRole}")
	private String defaultRole;


	@Override
	public BaseResult<UserDto> update(UserDto userDto) throws UserException {
		User user = new User();
		try {
			super.commonBuild(user, userDto);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UserException();
		}
		userDao.updateByPrimaryKeySelective(user);
		UserDto newDto = new UserDto();
		newDto.setId(userDto.getId());
		return ResultUtils.build(newDto);
	}

	@Override
	public BaseResult<UserDto> save(UserDto userDto) throws UserException {
		String mac = "";
		try {
			String item = "增";
			try {
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			String empuid = userDto.getEmpuid();
			if (StringUtils.isEmpty(empuid)) {
				userDto.setEmpuid(null);
			}
			save(userDto, User.class, usersDao, (Object dest, Object src) -> this.buildEntity(dest, src), "empuid");

			if (StringUtils.isEmpty(empuid)) {
				userDto.setEmpid(userDto.getEmpuid());
				save(userDto, User.class, usersDao, (Object dest, Object src) -> this.buildEntity(dest, src), "empuid");
			} else {
				// 用户信息修改
				item = "改";
			}
			if (userDto.getDefaultpos() != null) {
				Map<String, String> map = new HashMap<String, String>();
				logger.info(userDto.getDefaultpos());
				map.put("userId", userDto.getEmpuid());
				map.put("orgId", userDto.getDefaultpos());
				map.put("ismainpost", "P");
				// 添加主岗前，先判断表里是否已经有主岗（每个用户只有一个主岗），如果有，先删除，在添加主岗，避免报错，影响后面的代码
				int row = iOrganizationManagerDao.judgePost(map);
				if (row > 0) {
					iOrganizationManagerDao.removeMainPost(map);
				}
				iOrganizationManagerDao.assignOrgsToUser(map);
			}
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
		UserDto newDto = new UserDto();
		newDto.setId(userDto.getId());
		return ResultUtils.build(newDto);
	}

	@Override
	public BaseResult<UsersDto> saveUser(UsersDto usersDto) throws UserException {
		try {
			String empuid =usersDto.getEmpuid();
			if(StringUtils.isEmpty(empuid)){
				usersDto.setEmpuid(usersDto.getEmpid());
			}
			UserDto userDto = new UserDto();
			userDto.setEmpuid(usersDto.getEmpuid());
			userDto.setEmpid(usersDto.getEmpid());
			userDto.setCnname(usersDto.getCnname());
			userDto.setEmail(usersDto.getEmail());
			userDto.setPhone(usersDto.getMobilephone());
			userDto.setMobilephone(usersDto.getMobilephone());
			userDto.setSex(usersDto.getSex().equals("1")?"男":"女");
			userDto.setDisplayname(usersDto.getCnname());
			userDto.setDefaultpos("1000");
			userDto.setIsDeleted("N");
			userDto.setRemark("0");
			IpcOutUserDto ipcOutUserDto = new IpcOutUserDto();
			ipcOutUserDto.setContacts(usersDto.getCnname());
			ipcOutUserDto.setContactspinyin(usersDto.getEmpuid());
			ipcOutUserDto.setCallPhone(usersDto.getMobilephone());
			ipcOutUserDto.setEmail(usersDto.getEmail());
			ipcOutUserDto.setState(1);
			ipcOutUserDto.setCompany(usersDto.getCnname());
			ipcOutUserDto.setMark("Y");
			ipcOutUserDto.setCompanyType("内部员工");
			ipcOutUserDto.setCompanyTypeId("NBYG");
			ipcOutUserDto.setPassword("password1");
			ipcOutUserDto.setNewTime(DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
			//保存用户
			try {
				ipcOutUserService.saveOrUpdate(ipcOutUserDto);
				User user = new User();
				super.commonBuild(user,userDto);
				logger.info(user.getEmpuid());
				save(userDto, User.class, userDao, (Object dest, Object src) -> this.buildEntity(dest, src), "empuid");
				roleegoService.insertAuthRole(user.getEmpuid(),defaultRole);
				//用户同步到bpm引擎  natasha库里的employee表
				user.setDataSourceForSaveUser(dataSourceForSaveUser);
				user.setDefaultpos(1000);
				userDao.saveUser(user);
			} catch (Exception e) {
				e.printStackTrace();
			}
			//把用户默认岗位插入bas_basic_orgnazition_user表
			Map<String, String> map = new HashMap<String, String>();
			logger.info(empuid);
			map.put("userId", empuid);
			map.put("orgId", "1000");
			map.put("ismainpost", "P");
			// 添加主岗前，先判断表里是否已经有主岗（每个用户只有一个主岗），如果有，先删除，在添加主岗，避免报错，影响后面的代码
			int row = iOrganizationManagerDao.judgePost(map);
			if (row > 0) {
				iOrganizationManagerDao.removeMainPost(map);
			}
			iOrganizationManagerDao.assignOrgsToUser(map);

			//同步用户岗位到bpm引擎中
			iOrganizationManagerDao.assignUsersToBpmOrg(map);
			UsersDto newDto = new UsersDto();
			newDto.setId(userDto.getId());
			return ResultUtils.build(newDto);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}

	}



	/**
	 * 根据用户id判断用户是否是超级管理员
	 *
	 * @param userid
	 * @return
	 */
	@Override
	public int judgeAdminByUserId(String userid) {
		return userDao.judgeAdminByUserId(userid);
	}

	@Override
	public void pushDataIDMSeven(UserDto userDto) {
		try {
			// 系统编码//IDM员工编码//员工姓名 //员工性别//员工手机号码
			// 办公电话//邮箱//是否外部员工 :1是，0否//1，在职，0离职//推送状态，1推送中，0未推送
			// 根据员工id查询
			List<String> sysCodes = userDao.getSystemcodeByUserId(userDto.getEmpuid());
			for (String syscode : sysCodes) {
				String status = "0";
				if ("1".equals(userDto.getStatus())) {
					status = "1";
				}
				// 根据系统编码，员工登录账号，
				int row = userDao.getNumBySysCodes(syscode, userDto.getEmpid(), status);
				if (row == 0) {
					String external = "1";
					SystemUserEsbDto sueDto = new SystemUserEsbDto();
					sueDto.setBusnphone(userDto.getPhone());
					sueDto.setCnname(userDto.getCnname());
					if (userDto.getType().equals("EMP")) {
						external = "0";
					}
					sueDto.setExternal(external);
					sueDto.setHomemail(userDto.getEmail());
					sueDto.setCreateon(new Date());
					sueDto.setOperate("0");
					sueDto.setPhone(userDto.getMobilephone());
					sueDto.setSex(userDto.getSex());
					sueDto.setStatus(status);
					sueDto.setPushnum(0);
					sueDto.setSyscode(syscode);
					sueDto.setUserid(userDto.getEmpid());
					save(sueDto, SystemUserEsb.class, systemUserEsbDao);
					// userDao.insertSystemUser(syscode,userDto.getEmpid(),userDto.getCnname(),userDto.getSex(),userDto.getPhone()
					// ,userDto.getMobilephone(),userDto.getEmail(),external,userDto.getStatus(),"0");
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 用户初始化推送
	 *
	 * @throws UserException
	 */

	@Override
	public void initializationUserPush() throws UserException {
		try {
			QueryDTO query = new QueryDTO();
			query.setPageSize(10000);
			Example example = buildQuery(query, User.class);
			PageCriteria<UserDto> pages = query(query, () -> userDao.initializationUserPush(example), UserDto.class);
			String empuid = null;
			int iden = 1;
			List<UserDto> list = pages.getList();
			for (UserDto userDto : list) {
				callIDM(userDto, empuid, iden);
			}

		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new UserException();
		}

	}

	@Override
	public BaseResult<UserDto> delete(String id) throws UserException {
		userDao.deleteByPrimaryKey(id);
		return ResultUtils.build();
	}

	@Override
	public BaseResult<PageCriteria<UserDto>> query(QueryDTO query) throws UserException {
		//添加过滤条件，默认获取内部用户（eid:  0代表内部  1代表外部)
		query.addFilter(new FilterDTO("remark","1","EQ","S"));
		try {
			Example example = buildQuery(query, User.class);
			PageCriteria<UserDto> result = query(query, () -> userDao.selectByExample(example), UserDto.class,
					(Object dest, Object src) -> this.buildDTO(dest, src));
			BaseResult<PageCriteria<UserDto>> baseResult = ResultUtils.build(result);
			PageCriteria<UserDto> pageCriteria = baseResult.getData();
			List<UserDto> list = pageCriteria.getList();
			for(UserDto userDto:list) {
				String pname = userDao.getPName(userDto.getEmpuid());
				if(null==pname) {
					pname="";
				}
				userDto.setPname(pname);
			}
			pageCriteria.setList(list);
			baseResult.setData(pageCriteria);
			return baseResult;
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
	}

	@Override
	public BaseResult<UserDto> get(String id) throws BasException {
		UserDto result = get(UserDto.class, id, userDao);
		return ResultUtils.build(result);
	}
	@Override
	public BaseResult<UsersDto> getUser(String id) throws BasException {
		UsersDto result = get(UsersDto.class, id, usersDao);
		return ResultUtils.build(result);
	}

	@Override
	public boolean saveUserToBpm(UserToBpmDto userToBpmDto){
		User user = new User();
		String isDeleted =userToBpmDto.getIsDeleted();
		if(StringUtils.isEmpty(isDeleted)){
			userToBpmDto.setIsDeleted("N");
		}
		userToBpmDto.setDefaultpos(Integer.parseInt(defaultpos));
		userToBpmDto.setDeptid(deptid);
		try {
			super.commonBuild(user,userToBpmDto);
			user.setDataSourceForSaveUser(dataSourceForSaveUser);
			userDao.saveUserToBpm(user);
			//同步用户岗位
			Map<String, String> map = new HashMap<String, String>();
			map.put("userId", userToBpmDto.getEmpuid());
			map.put("orgId", defaultpos);
			map.put("ismainpost", "P");
			iOrganizationManagerDao.assignUsersToBpmOrg(map);
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}

		return false;
	}


	private void buildEntity(Object dest, Object src) {
		try {
			super.commonBuild(dest, src);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void buildDTO(Object dest, Object src) {
		try {
			UserDto userdto = (UserDto) dest;
			super.commonBuild(dest, src);

			OrganizationDto org = orgService.getUserDept(userdto.getEmpuid());
			if (org != null) {
				userdto.setDeptid(org.getId());
				userdto.setDeptname(org.getName());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// 导出用户
	@Override
	public FileObject exportsUser(QueryDTO query) throws UserException {

		return null;

	}

	// 导入用户
	@Override
	public FileObject importsUser(InputStream in) throws UserException {
		try {
			UserDto userDto = null;
			// 创建一个工作簿
			@SuppressWarnings("resource")
			HSSFWorkbook wb = new HSSFWorkbook(in);
			// 得到第一个工作表
			HSSFSheet sheet = wb.getSheetAt(0);
			HSSFRow row = null;
			// 获取总行数
			int rows = sheet.getLastRowNum();
			for (int i = 2; i <= rows; i++) {

				row = sheet.getRow(i);
				userDto = new UserDto();
				// 获得获得第i行第0列的 String类型对象
				Cell cell = row.getCell((short) 0);
				userDto.setCnname(cell.getStringCellValue());// 用户姓名
				cell = row.getCell((short) 1);
				userDto.setMobilephone(cell.getStringCellValue());// 电话号码
				if (StringUtils.isEmpty(userDto.getMobilephone())) {
					break;
				}
				int verificationPhoneOnly = userDao.verificationPhoneOnly(userDto.getMobilephone());
				if (verificationPhoneOnly > 0) {
					break;
				}
				// TODO 导入格式没有定好，这个地方还要写
				cell = row.getCell((short) 2);
				userDto.setEmail(cell.getStringCellValue());
				if (StringUtils.isEmpty(userDto.getEmail())) {
					break;
				}
				int verificationEmailOnly = userDao.verificationEmailOnly(userDto.getEmail());
				if (verificationEmailOnly > 0) {
					break;
				}
				cell = row.getCell((short) 3);
				userDto.setDefaultpos(cell.getStringCellValue());
				if (StringUtils.isEmpty(userDto.getDefaultpos())) {
					break;
				}
				DefaultPostDto dpDto = userDao.getPostDefalutByDefaultposid(userDto.getDefaultpos());
				if (dpDto == null) {
					break;
				}
				userDto.setDefaultposname(dpDto.getDefaultposname());
				userDto.setDeptid(dpDto.getDeptid());
				userDto.setDeptname(dpDto.getDeptname());
				cell = row.getCell((short) 4);
				String type = cell.getStringCellValue();
				if (StringUtils.isEmpty(type)) {
					break;
				}
				if (type.contains("内部")) {
					userDto.setType("EMP");
				} else if (type.contains("外部")) {
					userDto.setType("ExtUser");
				} else if (type.contains("系统")) {
					userDto.setType("SYSUser");
				} else if (type.contains("临时")) {
					userDto.setType("Temp");
				}
				userDto.setLockstatus("1");
				userDto.setStatus("1");
				save(userDto);
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (HeadlessException e) {
			e.printStackTrace();
		}
		System.setProperty("java.awt.headless", "true");
		FileObject fileObject = new FileObject();
		fileObject.setId("2");
		fileObject.setName("用户");
		fileObject.setSize(10000000);
		fileObject.setType(".xls");
		return fileObject;
	}

	@Override
	public PageCriteria<UserDto> queryUsersInOrg(QueryDTO query) throws UserException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public PageCriteria<UserDto> getUsersByNotBen(QueryDTO query) throws UserException {
		try {
			String userid = UserUtils.getCurrentUser();
			Example example = buildQuery(query, User.class);
			return query(query, () -> userDao.getUsersByNotBen(example, userid), UserDto.class);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}

	}

	@Override
	public PageCriteria<UserDto> getParentUserById(QueryDTO query) throws UserException {
		try {
			String userid = query.getAttr().get("userid");
			Example example = buildQuery(query, User.class);
			return query(query, () -> userDao.getParentUserById(example, userid), UserDto.class);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
	}

	// 查询组织下的所有用户
	@Override
	public List<UserDto> getAllUserByOrg(String orgid) throws UserException {
		List<User> entityList = userDao.getAllUserByOrg(orgid);
		List<UserDto> dtoList = new ArrayList<UserDto>();

		for (User user : entityList) {
			UserDto userDto = new UserDto();
			try {
				super.commonBuild(userDto, user);
				dtoList.add(userDto);
			} catch (Exception e) {

				e.printStackTrace();
			}
		}
		return dtoList;
	}

	// 查询当前用户下的所有用户
	@Override
	public PageCriteria<UserDto> getUsersByUser(QueryDTO query) throws UserException {
		try {
			String userid = UserUtils.getCurrentUser();
			Example example = buildQuery(query, User.class);
			return query(query, () -> userDao.getUsersByUser(example, userid), UserDto.class);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
	}

	/**
	 * 修改idm密码
	 *
	 * @param userId
	 * @param oldPassword
	 * @param newPassword
	 * @throws UserException
	 */
	@Override
	public SaveResultDTO changePassword(String userId, String oldPassword, String newPassword) throws UserException {
		SaveResultDTO sDto = new SaveResultDTO();
		try {
			if (!StringUtils.isEmpty(userId) && !StringUtils.isEmpty(newPassword)
					&& !StringUtils.isEmpty(oldPassword)) {

			} else {
				sDto.setMessage("账号、旧密码、新密码不能为空");
				sDto.setResult("1");
				return sDto;
			}
		} catch (Exception e) {
			throw new UserException(e);
		}
		return sDto;
	}

	@Override
	public SaveResultDTO modifyPasswordByPhone(Map<String, String> map) {
		SaveResultDTO saveResultDTO = new SaveResultDTO();
		String phone = map.get("phone");
		String newPassword = map.get("newPassword");
		String code = map.get("code");
		// 通过手机号码校验验证码
		CodeChange cc = userDao.checkCodeByPhoneAndCode(phone, code);
		if (cc != null) {
			Date date = new Date();
			long newTime = date.getTime() / 1000 / 60;
			long startTime = cc.getStarttime().getTime() / 1000 / 60;
			if ((newTime - startTime) < 5) {
				if (!StringUtils.isEmpty(phone) && !StringUtils.isEmpty(newPassword)) {
					try {
						// 通过手机号码获取到当前用户信息
						User user = userDao.getUserByPhone(phone, null);
						if (user != null) {
							String result = "0";
							userDao.deleteCodeById(cc.getId());
							if (result.equals("0")) {
								saveResultDTO.setResult("0");
								saveResultDTO.setMessage("修改密码成功！");
							} else {
								saveResultDTO.setResult("1");
								saveResultDTO.setMessage("修改密码失败！");
							}
						} else {
							saveResultDTO.setResult("1");
							saveResultDTO.setMessage("手机号码：" + phone + " 未绑定用户");
						}
					} catch (Exception e) {
						saveResultDTO.setResult("1");
						saveResultDTO.setMessage(e.getMessage());
						e.printStackTrace();
					}
				} else {
					saveResultDTO.setResult("1");
					saveResultDTO.setMessage("手机号码或新密码不能为空！");
				}
			} else {
				// 根据id删除验证码数据
				userDao.deleteCodeById(cc.getId());
				saveResultDTO.setResult("1");
				saveResultDTO.setMessage("验证码已过期");
			}
		} else {
			saveResultDTO.setMessage("验证码不正确");
			saveResultDTO.setResult("1");
		}
		return saveResultDTO;
	}

	@Override
	public SaveResultDTO modifyPasswordByEmail(Map<String, String> map) {
		SaveResultDTO saveResultDTO = new SaveResultDTO();
		String email = map.get("email");
		String newPassword = map.get("newPassword");
		String code = map.get("code");
		// 通过邮箱校验验证码
		CodeChange cc = userDao.checkCodeByPhoneAndCode(email, code);
		if (cc != null) {
			Date date = new Date();
			long newTime = date.getTime() / 1000 / 60;
			long startTime = cc.getStarttime().getTime() / 1000 / 60;
			if ((newTime - startTime) < 5) {
				if (!StringUtils.isEmpty(newPassword) && !StringUtils.isEmpty(email)) {
					try {
						User user = userDao.getUserByPhone(null, email);
						if (user != null) {
							String result = "0";
							userDao.deleteCodeById(cc.getId());
							if (result.equals("0")) {
								saveResultDTO.setResult("0");
								saveResultDTO.setMessage("修改密码成功！");
							} else {
								saveResultDTO.setResult("1");
								saveResultDTO.setMessage("修改密码失败！");
							}
						} else {
							saveResultDTO.setResult("1");
							saveResultDTO.setMessage("邮箱：" + email + " 尚未绑定用户");
						}
					} catch (Exception e) {
						saveResultDTO.setResult("1");
						saveResultDTO.setMessage(e.getMessage());
						e.printStackTrace();
					}
				} else {
					saveResultDTO.setResult("1");
					saveResultDTO.setMessage("邮箱或新密码不能为空！");
				}
			} else {
				userDao.deleteCodeById(cc.getId());
				saveResultDTO.setResult("1");
				saveResultDTO.setMessage("验证码已过期");
			}
		} else {
			saveResultDTO.setResult("1");
			saveResultDTO.setMessage("验证码不正确");
		}
		return saveResultDTO;
	}

	@Override
	public AccountDispose getAccountMappingByAccAndSyscode(String account, String syscode) throws UserException {
		try {
			return userDao.getAccountMappingByAccAndSyscode(account, syscode);
		} catch (Exception e) {
			throw new UserException();
		}
	}

	@Override
	public boolean deleteAccountByAccAndSyscode(String account, String syscode) {
		try {
			return userDao.deleteAccountByAccAndSyscode(account, syscode);
		} catch (Exception e) {
			return false;
		}
	}

	@Override
	public boolean validationCodeValidity(String account, String code) throws UserException {
		boolean flag = true;
		try {
			CodeChange cc = userDao.checkCodeByPhoneAndCode(account, code);
			if (cc != null) {
				Date date = new Date();
				long newTime = date.getTime() / 1000 / 60;
				long startTime = cc.getStarttime().getTime() / 1000 / 60;
				if ((newTime - startTime) < 5) {
					flag = true;
				} else {
					flag = false;
				}
				userDao.deleteCodeById(cc.getId());
			} else {
				return false;
			}
		} catch (Exception e) {
			throw new UserException();
		}
		return flag;
	}

	@Override
	public UserDto getUserByMobile(String mobile) throws UserException {
		UserDto userDto = null;
		List<User> userList = userDao.getUserByMobile(mobile);
		if (userList.size() > 0) {
			userDto = new UserDto();
			try {
				super.commonBuild(userDto, userList.get(0));
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return userDto;
	}

	@Override
	public PageCriteria<UserDto> queryList(QueryDTO query, String logid, int flag) throws UserException {
		try {
			Example example = buildQuery(query, User.class);
			if (flag > 0) {// 超级管理员
				return query(query, () -> userDao.getAllUserByLogid(example), UserDto.class);
			} else {
				return query(query, () -> userDao.getUserByLogid(example, logid), UserDto.class);
			}
		} catch (Exception e) {
			throw new UserException();
		}
	}

	@Override
	public String getAndPostByUserid(String empuid) throws UserException {
		String postr = "";
		try {
			List<String> andPosts = userDao.getAndPostByUserid(empuid);
			if (andPosts != null) {
				for (int i = 0; i < andPosts.size(); i++) {
					if (i == 0) {
						postr = andPosts.get(i);
					} else {
						postr += "," + andPosts.get(i);
					}
				}
				return postr;
			} else {
				return postr;
			}
		} catch (Exception e) {
			throw new UserException();
		}
	}

	@Override
	public UserDto getUserInfo(String userId) {
		return userDao.getUserInfo(userId);
	}

	@Override
	public List<OrganizationDto> getPostInfo() {
		return userDao.getPostInfo();
	}

	@Override
	public void resetPassword(List<String> list) throws UserException {
		// TODO Auto-generated method stub

	}

	@Override
	public void callIDM(UserDto userDto, String empuid, int iden) throws UserException {
		// TODO Auto-generated method stub

	}

	@Override
	public void disableAccount(String userid) throws UserException {
		userDao.disableAccount(userid);
	}

	@Override
	public void enableAccount(String userid) throws UserException {
		userDao.enableAccount(userid);
	}

	@Override
	public User getUserByEmpid(String empid) {
		// TODO Auto-generated method stub
		User user = new User();
		List<User> userlist = userDao.getUserByEmpid(empid);
		if (null != userlist && userlist.size() > 0) {
			user = userlist.get(0);
		}
		return user;
	}

	@Override
	public boolean updateToLdap(UserDto userDto, Map<String, String> map) throws UserException {
		LdapPersonDto account = new LdapPersonDto();
		try {
			account.setEmpNo(userDto.getEmpuid());
			account.setId(userDto.getEmpuid());
			account.setMail(userDto.getEmail());
			account.setMap(map);
			account.setMobilephone(userDto.getMobilephone());
			account.setName(userDto.getCnname());
			account.setState("1");
			account.setType("saas");
			account.setAccount(userDto.getEmpuid());
			ldapPersonService.update(account);
			return true;
		} catch (LdapException e) {
			return false;
		}
	}

	@Override
	public boolean saveToLdap(UserDto userDto, Map<String, String> map, String password) throws UserException {
		LdapPersonDto account = new LdapPersonDto();
		try {
			account.setEmpNo(userDto.getEmpuid());
			account.setId(userDto.getEmpuid());
			account.setMail(userDto.getEmail());
			account.setMap(map);
			account.setMobilephone(userDto.getMobilephone());
			account.setName(userDto.getCnname());
			account.setState("1");
			account.setType("saas");
			account.setUserPassword(password);
			account.setAccount(userDto.getEmpuid());
			ldapPersonService.create(account);
			return true;
		} catch (LdapException e) {
			return false;
		}
	}

	@Override
	public boolean modifyPasswordByVerificationCode(PasswordInfoDto passwordInfoDto) throws BasException {
		String way = passwordInfoDto.getWay();
		String action = "01";
		try {
			if (way == null) {
				throw new BasException("邮箱不能为空！");
			}
			List<VerificationCodeDto> verificationCodeDtoList = verificationCodeService.getByWayAndAction(way, action);
			if (verificationCodeDtoList.size() <= 0) {
				throw new BasException("验证码错误，请重新输入");
			}
			VerificationCodeDto verificationCodeDto = verificationCodeDtoList.get(0);
			String checkcode = verificationCodeDto.getCheckcode();
			String verificationcode = verificationCodeDto.getVerificationcode();
			if (verificationcode == null || !verificationcode.equals(passwordInfoDto.getVerificationcode())) {
				throw new BasException("验证码错误，请重新输入");
			}
			String newpassword = passwordInfoDto.getNewpassword();
			String userId = verificationCodeDto.getEmpuid();
			// 修改密码
//			ldapPersonService.resetPassword(userId, newpassword);
		} catch (BasException e) {
			throw new BasException(e.getMessage());
		}

		return false;
	}

	@Override
	public boolean modifyPasswordByOldPassword(PasswordInfoDto passwordInfoDto) throws BasException {
		try {
			String oldPassword = passwordInfoDto.getOldpassword();
			String newpassword = passwordInfoDto.getNewpassword();
			String userId = passwordInfoDto.getEmpuid();
			// 修改密码
			ldapPersonService.resetPassword(userId, oldPassword, newpassword);
			// 修改密码完成后，设置首次登陆为否
			try {
				userDao.updateFirstlogin("N", userId);
			} catch (Exception e) {
				// TODO: handle exception
			}
		} catch (BasException e) {
			String message = e.getMessage();
			if ("User authenticate failed".equals(message)) {
				message = "旧密码验证失败！";
			} else {
				message = "密码修改失败！";
			}
			throw new BasException(message);
		}
		return false;
	}

	@Override
	public BaseResult<UserDto> batchForbidden(UserDto userDto) throws UserException {
		try {
			String[] displaynames = userDto.getDisplayname().split(",");
			String[] ids = userDto.getEmpuid().split(",");
			for (int i = 0; i < ids.length; i++) {
				userDto.setIsDeleted(userDto.getIsDeleted());
				userDto.setEmpuid(ids[i]);
				userDto.setDisplayname(displaynames[i]);
				save(userDto, User.class, userDao, (Object dest, Object src) -> this.buildEntity(dest, src), "empuid");
			}

		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
		return ResultUtils.build();
	}

	@Override
	public String getEmpuidByEmpid(String empid) {
		// TODO Auto-generated method stub
		return userDao.getEmpuidByEmpid(empid);
	}

	@Override
	public boolean loginVerification(LoginVerificationDto loginVerificationDto) throws BasException {
		try {
			String businesstype = loginVerificationDto.getBusinesstype();
			String object = loginVerificationDto.getUsername();
			String password = loginVerificationDto.getPassword();
			List<Map<String, Object>> user = userDao.getEmployeesByMobilephoneEmpuidEmail(object);
			if (user.size() > 0) {
				String is_deleted = user.get(0).get("IS_DELETED").toString();
				String empuid = user.get(0).get("EMPUID").toString();
				String businesstypes = "";
				if (user.get(0).get("businesstype") == null) {
					if ("test02".equals(empuid)) {
						businesstypes = "00";
						} else {
						businesstypes = "01";
					}
				} else {
					businesstypes = user.get(0).get("businesstype").toString();
				}

				if ("Y".equals(is_deleted)) {
					throw new BasException("账号已禁用，请联系管理员");
				} else {
					if ("00".equals(businesstypes) || businesstype.equals(businesstypes)) {
						boolean isAuthenticated =true;
						//验证有户名和密码
						//isAuthenticated = ldapPersonService.loginVerification(empuid, password);
						if (!isAuthenticated) {
							throw new BasException("账号密码错误");
						} else {
							return true;
						}
					} else {
						throw new BasException("账号不存在");
					}
				}
			} else {
				throw new BasException("账号不存在");
			}
		} catch (BasException e) {
			throw new BasException(e.getMessage());
		}

	}


	@Override
	public PageInfo<InnerUserDto> selectPage(InnerUserPageDto innerUserPageDto) {
		PageHelper.startPage(innerUserPageDto.getPagenum(), innerUserPageDto.getPagesize());
		List<InnerUserDto> innerUserDtoList=userDao.selectPage(innerUserPageDto);
		PageInfo<InnerUserDto> pageInfo = new PageInfo<>(innerUserDtoList);
		return pageInfo;
	}



}